-- Exit if oUF isn't loaded
if not oUF then return end
--[[
Examples:

	-- Texture
	self.AFK = self:CreateTexture(nil, "OVERLAY")
	self.AFK:SetWidth(10)
	self.AFK:SetHeight(10)
	self.AFK:SetPoint("BOTTOM", self, "TOP")
	self.AFK:SetTexture(1, 0, 0)
	
OR
	-- Text, with timer
	self.AFK = self:CreateFontString(nil, "OVERLAY")
	self.AFK:SetFont("Fonts\\ARIALN.TTF", 10, "")
	self.AFK:SetWidth(10)
	self.AFK:SetHeight(10)
	self.AFK:SetPoint("BOTTOM", self, "TOP")
	self.AFK.fontFormat = "|cFFFF0000AFK |cFFFFFFFF%s:%s"
	
Replace self.AFK with self.DND to get DND status.
If you want to combine texture and text,
make sure that the function self.XXX.SetText(self, text) exists.
--]]

local objects = { }
local types = { "AFK", "DND" }
local timer = {
	AFK = { },
	DND = { },
	obj = { }
}

local OnUpdate, OnEvent
local UpdateFrame = CreateFrame("Frame", nil, UIParent)

local SecToMin = function(sec)
	local min = math.floor(sec/60)
	      sec = math.floor(sec-(min*60))
	if sec < 10 then sec = "0"..sec end
	return min, sec
end

local UpdateTimerText = function(obj, typ)
	local start = timer[typ][ UnitName(obj:GetParent().unit) or "xxx" ]
	if not start or not obj:IsShown() then
		obj:Hide()
		timer.obj[obj] = nil
		return
	end
	
	local min, sec = SecToMin(GetTime() - start)
	obj:SetText(obj.fontFormat:format(min, sec))
end

local nextUpdate = GetTime() + 3
local OnUpdateTexts = function()
	if nextUpdate > GetTime() then return end
	nextUpdate = GetTime() + 1

	local any = false
	for obj,typ in pairs(timer.obj) do
		any = true
		UpdateTimerText(obj, typ)
	end
	
	if not any then UpdateFrame:Hide() end
end

-- Only run at startup
OnUpdate = function()
	if nextUpdate > GetTime() then return end
	nextUpdate = GetTime() + 1
	
	-- Wait until we leave combat
	if InCombatLockdown() then
		return
	end
	
	for _,obj in pairs(objects) do
		if UnitExists(obj.unit) then
			OnEvent(_, "PLAYER_FLAGS_CHANGED", obj.unit)
		end
	end
	
	-- Switch updater
	UpdateFrame:SetScript("OnUpdate", OnUpdateTexts)
end

local ActivateText = function(obj, typ)
	local fs = obj[typ]
	fs:Show()
	timer.obj[fs] = typ

	-- Activate update frame
	if not UpdateFrame:IsShown() then
		UpdateFrame:Show()
	end
end

local RepointTimers = function()
	-- Clear table
	local any = false
	for obj,_ in pairs(timer.obj) do
		any = true
		obj:Hide()
		timer.obj[obj] = nil
	end
	if not any then return end
	
	for _,obj in ipairs(objects) do
		for _,typ in ipairs(types) do
			if obj[typ] and timer[typ][UnitName(obj.unit)] then
				ActivateText(obj, typ)
			end
		end
	end
end

local AddTimerByUnit = function(unit, typ)
	for _,obj in ipairs(objects) do
		if obj[typ] and UnitIsUnit(obj.unit, unit) then
			ActivateText(obj, typ)
		end
	end
	return true
end

local DelayFrame = CreateFrame("Frame")
OnEvent = function(_, event, unit)
	if event == "PLAYER_FLAGS_CHANGED" then
		local name = UnitName(unit)
		timer.AFK[name] = UnitIsAFK(unit) and (timer.AFK[name] or (AddTimerByUnit(unit, "AFK") and GetTime()))
		timer.DND[name] = UnitIsDND(unit) and (timer.DND[name] or (AddTimerByUnit(unit, "DND") and GetTime()))
	else
		local waitUntill = GetTime() + .5
		DelayFrame:SetScript("OnUpdate", function()
				if waitUntill > GetTime() then return end
				RepointTimers()
				DelayFrame:SetScript("OnUpdate", nil)
			end)
		
	end
end

UpdateFrame:RegisterEvent("PLAYER_FLAGS_CHANGED")
UpdateFrame:RegisterEvent("RAID_ROSTER_UPDATE")
UpdateFrame:SetScript("OnEvent", OnEvent)
UpdateFrame:SetScript("OnUpdate", OnUpdate)

local hook = function(object)
	if not object.AFK and not object.DND then return end
	tinsert(objects, object)
end

-- Append to all existing frames
for _,object in pairs(oUF.objects) do hook(object) end

-- Append to frames created later on
oUF:RegisterInitCallback(hook)
