-- Exit if oUF isn't loaded
if not oUF then return end
local print = function(m) DEFAULT_CHAT_FRAME:AddMessage(tostring(m)) end
-- Values are: Red,  Green  Blue  Alpha
local otherHealsColor =	{ 0, .781, 0, .3 }
local myOverHealColor = {
	[0]		= {   0,   1,   0,  .4 },
	[25]	= {   0,  .5,   0,  .4 },
	[50]	= {   1,   1,   0,  .5 },
	[75]	= {   1, .25,   0,  .5 },
	[100]	= {   1,   0,   0,  .6 }
}
local useArrow = true
local useTimeleft = true
local arrowColor =	{ 1, 1, 1, .9 }
local arrowTexture = "Interface\\AddOns\\oUF_HealEstimation\\arrow"

local oUF_HealEstimation = {}
local HealEstimation = LibStub("LibHealComm-3.0")
local playerName = UnitName("player")
local playerHealSize = 0
local playerHealLand = nil
local objects = { }

local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local floor = math.floor
local UnitHealth, UnitHealthMax = UnitHealth, UnitHealthMax
local SecureButton_GetUnit = SecureButton_GetUnit
local curHP, maxHP, percHP, preHeal, postHeal, unitName, unit

-- Adds server in front of name if unit isn't local
local fullUnitName = function(unit)
	local name, server = UnitName(unit)
	if server then name = name.."-"..server end
	return name
end

local horizontalUpdate = function(frame, i, percInc, percHP)
	local w = percInc * frame.Health:GetWidth()
	if (w == 0) then w = 0.001 end
    frame.HealEstimation.bars[i]:SetWidth(w)
	if (i == 1) then
		frame.HealEstimation.bars[1]:SetPoint("LEFT", frame.Health, "LEFT", frame.Health:GetWidth() * percHP, 0)
	end
end

local verticalUpdate = function(frame, i, percInc, percHP)
	local h = percInc * frame.Health:GetHeight()
	if (h == 0) then h = 0.001 end
    frame.HealEstimation.bars[i]:SetHeight(h)
	if (i == 1) then
		frame.HealEstimation.bars[1]:SetPoint("BOTTOM", frame.Health, "BOTTOM", 0, frame.Health:GetHeight() * percHP)
	end
end

local updatePlayerHealColor = function(maxHP, curHP, preHeal)
	-- Set overheal color
	local colorIndex = 0 -- Assume no overheal
	local hpMissing = maxHP - curHP - preHeal
	if hpMissing <= 0 then -- Full overheal
		colorIndex = 100
	else
		local percOH = hpMissing / playerHealSize
		if (percOH < 1) then
			colorIndex = 100 - ( floor((percOH + 0.125) * 4)*25 )
		end
	end
	return myOverHealColor[colorIndex]
end

-- Dispatch one heal to several frames
local updateHealEstimationBars = function(...)
	local incHeal = {}
	for i = 1, select("#", ...) do
		unitName = fullUnitName( select(i, ...) )
		if unitName then
		    curHP = UnitHealth(unitName)
		    maxHP = UnitHealthMax(unitName)
		    percHP = curHP / maxHP
	        preHeal, postHeal = HealEstimation:UnitIncomingHealGet(unitName, playerHealLand or (GetTime() + 3))
			preHeal = preHeal or 0
			postHeal = postHeal or 0
	        for _,object in pairs(objects) do
				local unit = SecureButton_GetUnit(object)
	            if unit and fullUnitName(unit) == unitName then
					object.HealEstimation.SetHPValue(object, 1, preHeal/maxHP, percHP)
					
					object.HealEstimation.anyActive = false
					object.HealEstimation.preHeal = preHeal
					if preHeal > 0 then
						object.HealEstimation.bars[1]:Show()
						object.HealEstimation.anyActive = true
					else
						object.HealEstimation.bars[1]:Hide()
					end
					
					if object.HealEstimation.playerIsHealing then
						object.HealEstimation.SetHPValue(object, 2, playerHealSize/maxHP, percHP)
						object.HealEstimation.bars[2].texture:SetVertexColor(
							unpack(updatePlayerHealColor(maxHP, curHP, preHeal))
						)
						object.HealEstimation.bars[2]:Show()
						object.HealEstimation.anyActive = true
						postHeal = postHeal - playerHealSize
					else
						object.HealEstimation.bars[2]:Hide()
					end
					
					if postHeal > 0 then
						object.HealEstimation.SetHPValue(object, 3, postHeal/maxHP, percHP)
						object.HealEstimation.bars[3]:Show()
						object.HealEstimation.anyActive = true
					else
						object.HealEstimation.bars[3]:Hide()
					end
	            end
	        end
		end
	end
end

-- Update estimators when HP changes
local unitHealthUpdate = function(object, unit)
	if not object.HealEstimation.anyActive then return end

	if UnitIsDead(unit) or UnitIsGhost(unit) or not UnitIsConnected(unit) or not(object.HealEstimation.playerIsHealing or select(3, HealEstimation:UnitIncomingHealGet(fullUnitName(unit), (GetTime() + 4)))) then
		-- Hide bars
		object.HealEstimation.bars[1]:Hide()
		object.HealEstimation.bars[2]:Hide()
		object.HealEstimation.bars[3]:Hide()
		object.HealEstimation.anyActive = false
	else
		curHP = UnitHealth(unit)
		maxHP = UnitHealthMax(unit)
		percHP = curHP / maxHP
		if (object.HealEstimation.playerIsHealing) then
			object.HealEstimation.bars[2].texture:SetVertexColor(
				unpack(updatePlayerHealColor(maxHP, curHP, object.HealEstimation.preHeal	))
			)
		end
		object.HealEstimation.SetHPValue(object, 1, object.HealEstimation.preHeal/maxHP, percHP)
	end
end

local updateTimer = function(f)
	if not playerHealLand then return end
	if (playerHealLand - GetTime() < 0) then
		f.text:SetText()
	elseif (playerHealLand - GetTime() > 1) then
		f.text:SetText(floor(playerHealLand - GetTime() + .7))
	else
		f.text:SetText("."..floor((playerHealLand - GetTime())*10))
	end
end

local function hook(frame)
	if frame.ignoreHealEstimation or frame.ignoreIncomingHeal or not frame.Health then return end
	
	tinsert(objects, frame)
	
	local heb = { }
	heb.bars = { }
	local last = frame.Health
	
	for i = 1, 3 do
		local bar = CreateFrame("Frame", nil, frame)
		bar:SetFrameLevel(10)
		bar.texture = bar:CreateTexture(nil, "BORDER")
		bar.texture:SetTexture(frame.Health:GetStatusBarTexture():GetTexture())
		bar.texture:SetTexCoord(frame.Health:GetStatusBarTexture():GetTexCoord())
	    bar.texture:SetVertexColor(unpack(otherHealsColor))
		bar.texture:SetAllPoints(bar)
	    bar:SetParent(frame)
		
		if ( frame.Health:GetOrientation() ~= "VERTICAL" ) then
			bar:SetHeight(frame.Health:GetHeight()) -- Same size as the Health bar
		    bar:SetWidth(20)
			bar:SetPoint("LEFT", last, "RIGHT") -- Assume the bar grows towards right
		else
			bar:SetHeight(20)
		    bar:SetWidth(frame.Health:GetWidth()) -- Same size as the Health bar
			bar:SetPoint("BOTTOM", last, "TOP") -- Assume the bar grows upwards
		end
	    bar:Hide()
		last = bar
		
		heb.bars[i] = bar
	end
	
	local f = heb.bars[2]
	if useTimeleft then
		f.text = f:CreateFontString(nil, "OVERLAY")
		f.text:SetFont([[Fonts\ARIALN.TTF]], 12, "Outline")
		f.text:SetPoint("CENTER", f, "CENTER")
		f:SetScript("OnShow", function()
			f:SetScript("OnUpdate", function() updateTimer(f) end)
		end)
		
		f:SetScript("OnHide", function()
			f:SetScript("OnUpdate", nil)
		end)
	end
	
	if useArrow then
		heb.arrow = f:CreateTexture(nil, "OVERLAY")
		heb.arrow:SetTexture(arrowTexture)
		heb.arrow:SetVertexColor(unpack(arrowColor))
		local size = min(frame.Health:GetHeight()/2.5, frame.Health:GetWidth()/2.5)
		heb.arrow:SetHeight(size)
		heb.arrow:SetWidth(size)
		if ( frame.Health:GetOrientation() ~= "HORIZONTAL" ) then
			heb.arrow:SetTexCoord( 1, 1,
								   0, 1,
								   1, 0,
								   0, 0 )
		end
		heb.arrow:SetPoint("CENTER", frame.Health, "TOPRIGHT")
	end
			
	if ( frame.Health:GetOrientation() == "HORIZONTAL" ) then
		heb.SetHPValue = horizontalUpdate
	else
		heb.SetHPValue = verticalUpdate
	end

    frame.HealEstimation = heb

	local OldPostHP = frame.PostUpdateHealth
	frame.PostUpdateHealth = function(...)
		if OldPostHP then OldPostHP(...) end
        unitHealthUpdate(frame, fullUnitName(frame.unit)) -- When HP is updated, update color and hide outdated bars
	end
end

-- Append to all existing frames
for object in pairs(oUF.objects) do hook(object) end

-- Append to frames created later on
oUF:RegisterInitCallback(hook)

--set up LibHealEstimation callbacks
function oUF_HealEstimation:HealComm_DirectHealStart(event, healerName, healSize, endTime, ...)
	if healerName == playerName then
		for i = 1, select("#", ...) do
			unitName = select(i, ...)
	        -- Mark all frames we are healing
	        for _,object in pairs(objects) do
				unit = SecureButton_GetUnit(object)
	            if unit and fullUnitName(unit) == unitName then
	                object.HealEstimation.playerIsHealing = true
	            end
	        end
		end
		playerHealLand = endTime
		playerHealSize = healSize
	end
    updateHealEstimationBars(...)
end

function oUF_HealEstimation:HealComm_DirectHealUpdate(event, healerName, healSize, endTime, ...)
	if healerName == playerName then
		playerHealLand = endTime
		playerHealSize = healSize
	end
    updateHealEstimationBars(...)
end

function oUF_HealEstimation:HealComm_DirectHealStop(event, healerName, healSize, succeeded, ...)
	if healerName == playerName then
		-- Clear all player is healing flags
		for _,object in pairs(objects) do
			if object.HealEstimation then
				object.HealEstimation.playerIsHealing = false
			end
		end
		playerHealLand = nil
		playerHealSize = 0
	end
    updateHealEstimationBars(...)
end

function oUF_HealEstimation:HealComm_HealModifierUpdate(event, unit, targetName, healModifier)
    updateHealEstimationBars(unit)
end

HealEstimation.RegisterCallback(oUF_HealEstimation, "HealComm_DirectHealStart")
HealEstimation.RegisterCallback(oUF_HealEstimation, "HealComm_DirectHealUpdate")
HealEstimation.RegisterCallback(oUF_HealEstimation, "HealComm_DirectHealStop")
HealEstimation.RegisterCallback(oUF_HealEstimation, "HealComm_HealModifierUpdate")
