if not oUF then return end

-- Customize these
local onlyShowForPlayer = true
local maxAlpha = .8
local patterns = {
	Heal = "|cFF00FF00+%d",
	OverHeal = "|cFF00FF00+%d |cFF55FF55<%d>",
	Damage = "|cFFFF0000%d",
	Immune = "|cFFFF0000IMMUNE",
	Absorbe = "|cFFFFFF00(Absorbe)",
	Interrupt = "|cFFFF0000Interrupt",
	Dispel = "|cFF00FF00-Dispel",
	Stolen = "|cFF00FF00-Stolen",
	Buffed = "|cFF0000FF+Buffed",
}

local fade_in = COMBATFEEDBACK_FADEINTIME
local fade_out = COMBATFEEDBACK_FADEOUTTIME
local fade_hold = COMBATFEEDBACK_HOLDTIME

-- Don't change below this line
local playerGUID = UnitGUID( "player" )
local keepAlive = false
local e
local visual = { }
local objects = { }

local initOnUpdate
local holdOnUpdate

local function updateCombatLogText(frame, unit)
	local fontMulti = 1
	local pattern = "Normal"
	local text
	local pcf = frame.HealingFeedback
	
	if arg2:find("_HEAL") then
		
		if arg14 then -- Crit
			fontMulti = 1.2
		end
		if arg13 and arg13 > 0 then
			text = string.format(patterns["OverHeal"], arg12-arg13, arg13)
		else
			text = string.format(patterns["Heal"], arg12)
		end
	
	elseif arg2:find("_INTERRUPT") then
		text = patterns["Interrupt"]
		
	elseif arg2:find("_DISPEL") then
		text = patterns["Dispel"]
		
	elseif arg2:find("_STOLEN") then
		text = patterns["Stolen"]

	else
		return -- Exit
		
	end
	
	pcf:Show()
	pcf:SetText(text)
	pcf:SetFont(pcf.oFont, pcf.oFontHeight*fontMulti, pcf.oFontFlags)
	visual[pcf] = GetTime()
	initOnUpdate()
end

local function onCombatLogEvent()
	if not playerGUID then -- GUID is sometimes nil @ load, but not at reload
		playerGUID = UnitGUID( "player" )
	end
	if (onlyShowForPlayer and arg3 ~= playerGUID) then -- Skipp all but our guild
		return
	end
	
	local unit
	for _,object in pairs(objects) do
		unit = SecureButton_GetUnit(object)
		if unit and UnitGUID(unit) == arg6 then
			updateCombatLogText(object, unit)
		end
	end
end

local function updateScript()
	if next(visual) == nil then
		holdOnUpdate()
		return
	end
	for pcf, startTime in pairs(visual) do
		local maxalpha = pcf.maxAlpha
		local elapsedTime = GetTime() - startTime
		local alpha = 0
		if ( elapsedTime < fade_in ) then
			alpha = maxalpha*(elapsedTime / fade_in)
		elseif ( elapsedTime < (fade_in + fade_hold) ) then
			alpha = maxalpha
		elseif ( elapsedTime < (fade_in + fade_hold + fade_out) ) then
			alpha = maxalpha - maxalpha*((elapsedTime - fade_hold - fade_in) / fade_out)
		else
			pcf:Hide()
			visual[pcf] = nil
		end
		pcf:SetAlpha(alpha)
	end	
end

initOnUpdate = function()
	if not keepAlive then
		keepAlive = true
		e:SetScript("OnUpdate", updateScript)
	end
end

holdOnUpdate = function()
	if keepAlive then
		keepAlive = false
		e:SetScript("OnUpdate", nil)
	end
end

do
	e = CreateFrame("Frame")
	e:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	e:SetScript("OnEvent", onCombatLogEvent)
end

local function hook(frame)
	if not frame.HealingFeedback then return end
	local font, fontHeight, fontFlags = frame.HealingFeedback:GetFont()
	frame.HealingFeedback.oFont = font
	frame.HealingFeedback.oFontHeight = fontHeight
	frame.HealingFeedback.oFontFlags = fontFlags
	frame.HealingFeedback.maxAlpha = frame.HealingFeedback.maxAlpha or maxAlpha
	tinsert(objects, frame)
end

for _, frame in ipairs(oUF.objects) do hook(frame) end
oUF:RegisterInitCallback(hook)
