-- Requires oUF
if not oUF or not oUF.Smurf then return end

local unit = oUF.HolySmurf.Config.units
local groups = oUF.HolySmurf.Config.groups
local onePix = UIParent:GetEffectiveScale()
local class = select(2, UnitClass("player"))

--[[

How to enable this file:
  You need to rename this file to custom.lua

Reason for using this file:
  Instead of editing unitconfig, groupconfig and templates,
  you can use this file to overwrite settings. It is loaded
  after all the other files, so normaly you would add settings
  or remove the default once.
  
  This file will NOT get overwritten when updating this layout,
  but if the layout changes, this might effect your frames.

--]]

---------------------------------
-- Example: 
---------------------------------
unit.player.castbar = nil      -- Remove the player castingbar

unit.raid.scale = 1.2          -- Set the scale raid up to 1.2
unit.raid.rangecheck = nil     -- Disable rangecheck on raid frames

-- Change the raidpet to have 5 px width, and the same height as the raid units.
-- Then anchor it to the right of the raid unit owning the pet, and change the
-- healthbar to be vertical oriented.
unit.raidpet.width = 5
unit.raidpet.height = unit.raid.height
unit.raidpet.anchorPoint = "LEFT"
unit.raidpet.anchorRelative = "RIGHT"
unit.raidpet.anchorOffsetX = onePix      -- 1px, allways good with some space
unit.raidpet.anchorOffsetY = 0           -- Reset to 0, usualy 1 px
unit.raidpet.element.health.height = unit.raidpet.height
unit.raidpet.element.health.orientation = "VERTICAL"
unit.raidpet.element.healthtext.anchorPoint = "BOTTOM"
unit.raidpet.element.healthtext.anchorRelative = "BOTTOM"
unit.raidpet.element.healthtext.anchorOffsetX = 0
unit.raidpet.element.healthtext.fontSize = 7

------------------------------------------------------------------
-- For groups, they are added with number indexes only. If you
-- haven't changed anything, the index order should be:
--   1  = Raid1
--   2  = Raid5 <= Note!
--   3  = Raid2
--   4  = Raid3
--   5  = Raid4
--   6  = Raid6
--   7  = Raid7
--   8  = Raid8
--   9  = PlayerTargets
--   10 = MainTank
--   11 = MainAssist
--   12 = Party
-- You should NOT clear a key by groups[3] = nil, unless you wish
-- to disable all the following groups. This brakes the ipairs
-- loope in smurf.lua
------------------------------------------------------------------

-- All raid members are showin in the same group, ordered by class
groups[1].manyAttributes = function() return "showRaid", true, "yOffSet", -onePix, "initial-scale", unit.raid.scale or 1 end

-- Clear the other groups
groups[2].manyAttributes = function() return "nameList", "", "showRaid", true end -- A blank list => Not shown
groups[3].manyAttributes = function() return "nameList", "", "showRaid", true end -- A blank list => Not shown
groups[4].manyAttributes = function() return "nameList", "", "showRaid", true end -- A blank list => Not shown
groups[5].manyAttributes = function() return "nameList", "", "showRaid", true end -- A blank list => Not shown
groups[6].manyAttributes = function() return "nameList", "", "showRaid", true end -- A blank list => Not shown
groups[7].manyAttributes = function() return "nameList", "", "showRaid", true end -- A blank list => Not shown
groups[8].manyAttributes = function() return "nameList", "", "showRaid", true end -- A blank list => Not shown


---------------------------------
-- Edit the TargetRangeCheck: 
---------------------------------
oUF.RangeCheckConfig = { }
if class == "MAGE" then
	oUF.RangeCheckConfig.rangeAlphaEnemy = {
		[0] = 1,      -- 0 and above to the next, use 100% alpha
		[42] = .3,    -- 42 yards and above, use 30% alpha
		[1000] = .2,
	}
elseif UnitName("player") == "Someone" then
	oUF.RangeCheckConfig.rangeAlphaEnemy = {
		[0] = 1,
		[37] = .3,
		[1000] = .2,
	}
else
	oUF.RangeCheckConfig.rangeAlphaEnemy = {
		[0] = 1,
		[31] = .3,
		[1000] = .2,
	}
end

oUF.RangeCheckConfig.rangeAlphaFriendly = {
	[0] = 1,      
	[41] = .3,    
	[1000] = .2,
}


