--[[

This file contains:
 * Generic text generating functions, like HP% of a given unit.
 * Standard values for each element

--]]
-- Requires oUF
if not oUF or not oUF.Smurf then return end

oUF.HolySmurf = { }
local Skin = oUF.HolySmurf

local BarTexture1 = "Interface\\AddOns\\oUF_HolySmurf\\textures\\PlainSmurf" -- HP Bars mainly
local BarTexture2 = "Interface\\AddOns\\oUF_HolySmurf\\textures\\Minimalist" -- Most others
local StripeTexture = "Interface\\AddOns\\oUF_HolySmurf\\textures\\SmurfStripe"
local MainFont = [[Fonts\ARIALN.TTF]]
local MainFontSize = 10
local onePix = UIParent:GetEffectiveScale()

-- Used in the TargetName function, appends type of mob
local classification = {
	worldboss	= "%s Boss",
	rareelite	= "%s+ Rare",
	elite		= "%s+",
	rare		= "%s Rare",
	normal		= "%s",
	trivial		= "%s",
}

local PowerName = {
	[0] = "MANA",
	[1] = "RAGE",
	[2] = "FOCUS",
	[3] = "ENERGY",
	[4] = "HAPPINESS",
	[5] = "RUNES",
	[6] = "RUNICPOWER"
}

----------------------------------
--	Setup our tables	--
----------------------------------
Skin.Func = { } -- Functions
Skin.Elem = { } -- Elements
Skin.Config = { } -- Config
Skin.Config.units = { } -- Units
Skin.Config.groups = { } -- Groupes

----------------------------------------------------------------------------
-- 		Text functions
----------------------------------------------------------------------------
-- Used to generate text for the various units
----------------------------------------------------------------------------
--	Helpers		--
----------------------------------
local ShortNumber = function(n)											-- number reformating																			
	if n >= 1e7 then
		return ('%.1fm'):format(n / 1e6):gsub('%.?0+([km])$', '%1')
	elseif n >= 1e6 then
		return ('%.2fm'):format(n / 1e6):gsub('%.?0+([km])$', '%1')
	elseif n >= 1e5 then
		return ('%.0fk'):format(n / 1e3):gsub('%.?0+([km])$', '%1')
	elseif n >= 1e3 then
		return ('%.1fk'):format(n / 1e3):gsub('%.?0+([km])$', '%1')
	else
		return n
	end
end

local NotAliveText = function(unit)
	if UnitIsDead(unit) then
		return "Dead"
	elseif UnitIsGhost(unit) then
		return "Ghost"
	elseif not UnitIsConnected(unit) then
		return "Offline"
	else
		return false
	end
end

----------------------------------
--	Power texts		--
----------------------------------
Skin.Func.PowerTextLeft = function(self, event, unit, bar, min, max)
	local notAlive = NotAliveText(unit)
	if notAlive or max <= 0 then
		self.PowerText:SetText()
	else
		self.PowerText:SetText(ShortNumber(min))
		if min==max then
			self.PowerText:SetTextColor( unpack(oUF.colors.power[ PowerName[ UnitPowerType(unit)] ]) )
		else
			self.PowerText:SetTextColor(1, 1, 1)
		end
	end
end

Skin.Func.PowerTextTarget = function(self, event, unit, bar, min, max)
	local notAlive = NotAliveText(unit)
	if notAlive or max <= 0 then
		self.PowerText:SetText()
	else
		self.PowerText:SetText(ShortNumber(min).." | "..ShortNumber(max))
		if min==max then
			self.PowerText:SetTextColor( unpack(oUF.colors.power[ PowerName[ UnitPowerType(unit)] ]) )
		else
			self.PowerText:SetTextColor(1, 1, 1)
		end
	end
end

----------------------------------
--	HP texts		--
----------------------------------
Skin.Func.HealthTextHPLeft = function(self, event, unit, bar, min, max)
	local notAlive = NotAliveText(unit)
	if notAlive then
		self.HealthText:SetText(notAlive)
	else
		self.HealthText:SetText(ShortNumber(min))
	end
end

Skin.Func.HealthTextHPPercent = function(self, event, unit, bar, min, max)
	local notAlive = NotAliveText(unit)
	if notAlive then
		self.HealthText:SetText(notAlive)
	else
		self.HealthText:SetText((math.floor(min/max*100)).."|cFFAAAAFF%")
	end
end

Skin.Func.HealthTextTarget = function(self, event, unit, bar, min, max)
	local notAlive = NotAliveText(unit)
	if notAlive then
		self.HealthText:SetText(notAlive)
	else
		self.HealthText:SetText("|cFFFFFFFF"..ShortNumber(min).." |cFFAAAAAA| |cFFFFFFFF"..ShortNumber(max).." |cFFAAAAAA| |cFF00CC00"..(math.floor((min or 1)/(max or 1)*100)).."%")
	end
end

Skin.Func.HealthTextHPMissing = function(self, event, unit, bar, min, max)
	local notAlive = NotAliveText(unit)
	if notAlive then
		self.HealthText:SetText(notAlive)
	elseif min~=max then
		self.HealthText:SetText(max-min)
	else
		self.HealthText:SetText(nil)
	end
end

----------------------------------
--	Nametexts		--
----------------------------------
Skin.Func.ShortName = function(self, event, unit)
	if(self.unit == unit and UnitExists(unit)) then
		local name = UnitName(unit)
		if string.len(name) > 4 then name = name:sub(1,4) end
		self.Name:SetText(name)
		local _, class = UnitClass(unit)
		self.Name:SetTextColor( unpack(oUF.colors.class[class]) )
	end
end

Skin.Func.TargetName = function(self, event, unit)
	if(self.unit == unit and UnitExists(unit)) then
		local name = UnitName(unit)
		local lvl = UnitLevel(unit)
		if lvl < 0 then lvl = "??" end
		if string.len(name) > 13 then name = name:sub(1,13) end
		if(UnitIsFriend(unit, "player") or UnitLevel(unit) < 0) then
			self.Name:SetTextColor(unpack(self.colors.reaction[UnitReaction(unit, 'player')] or self.colors.disconnected))
		else
			color = GetDifficultyColor(UnitLevel(unit))
			self.Name:SetTextColor(color.r, color.g, color.b)
		end
		self.Name:SetText('|cffFFFFFF' .. name .. '|r' .. ' ' .. format(classification[UnitClassification(unit)], lvl))
	end
end

----------------------------------
--	Bufflayout		--
----------------------------------
function Skin.Func.PostCreateAuraIcon(self, button, icons, index, debuff)
	button.cd:SetReverse()
	
	button.icon:SetDrawLayer("ARTWORK")
	button.icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	button.icon:ClearAllPoints()
	button.icon:SetPoint("TOPLEFT", button, "TOPLEFT", onePix*2, -(onePix*2))
	button.icon:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -(onePix*2), onePix*2)
	
	--[ [
	button.overlay:SetDrawLayer("BORDER")
	button.overlay:SetTexture(1, 1, 1, 1)
	button.overlay:ClearAllPoints()
	button.overlay:SetPoint("TOPLEFT", button, "TOPLEFT", onePix, -onePix)
	button.overlay:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -onePix, onePix)
	
	button.border = button:CreateTexture(nil, "BACKGROUND")
	button.border:SetTexture(0, 0, 0, 1)
	button.border:SetAllPoints(button)
	--]]
	--[[
	button.overlay:SetDrawLayer("BORDER")
	button.overlay:SetTexture(1, 1, 1, 1)
	button.overlay:ClearAllPoints()
	button.overlay:SetAllPoints(button)
	--]]
	
	button.count:SetFont(MainFont, math.floor(button:GetWidth()*.6), "Outline")
	button.count:ClearAllPoints()
	button.count:SetPoint("CENTER", button, "RIGHT", -1, 0)
end

----------------------------------
--	Castingbar		--
----------------------------------
Skin.Func.CastingBar = function(self)
	self.Castbar = CreateFrame("StatusBar")
	self.Castbar:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", insets ={left = -.8, right = -.8, top = -.8, bottom = -.8}})
	self.Castbar:SetBackdropColor(0, 0, 0)
	self.Castbar:SetWidth(self.db.width-10.8)
	self.Castbar:SetHeight(10)
	self.Castbar:SetStatusBarTexture(BarTexture2)	
	self.Castbar:SetStatusBarColor(unpack(oUF.colors.power.MANA))	 -- mana color[0]
	self.Castbar:SetParent(self)
	self.Castbar:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 10.8, .8)
	self.Castbar:SetMinMaxValues(1, 100)
	self.Castbar:SetValue(1)
	self.Castbar:Hide()  
	self.Castbar.bg = self.Castbar:CreateTexture(nil, "BORDER")
	self.Castbar.bg:SetAllPoints(self.Castbar)
	self.Castbar.bg:SetTexture(BarTexture2)
	self.Castbar.bg:SetAlpha(0.05)	
	self.Castbar.Spark = self:CreateTexture(nil, "OVERLAY")
	self.Castbar.Spark:SetParent(self.Castbar)
	self.Castbar.Spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	self.Castbar.Spark:SetBlendMode("ADD")
	self.Castbar.Spark:SetHeight(self.Castbar:GetHeight() * 2)
	self.Castbar.Spark:SetWidth(self.Castbar:GetHeight() * 2)
	self.Castbar.Time = self.Castbar:CreateFontString(nil, "OVERLAY")
	self.Castbar.Time:SetPoint("RIGHT", self.Castbar, -2, 0)
	self.Castbar.Time:SetFontObject(GameFontNormalSmall)
	self.Castbar.Time:SetTextColor(1, 1, 1)
	self.Castbar.Time:SetJustifyH("RIGHT")
	self.Castbar.Text = self.Castbar:CreateFontString(nil, "OVERLAY")
	self.Castbar.Text:SetPoint("LEFT", self.Castbar, 2, 0)
	self.Castbar.Text:SetWidth(self.db.width/2)
	self.Castbar.Text:SetFontObject(GameFontNormalSmall)
	self.Castbar.Text:SetTextColor(1, 1, 1)
	self.Castbar.Text:SetJustifyH"LEFT"
	self.Castbar.SafeZone = self.Castbar:CreateTexture(nil, "ARTWORK")
	self.Castbar.SafeZone:SetTexture(BarTexture2)
	self.Castbar.SafeZone:SetVertexColor(1, 0, 0, .8) -- Status bare safe area to the right
	self.Castbar.SafeZone:SetPoint("TOPRIGHT")
	self.Castbar.SafeZone:SetPoint("BOTTOMRIGHT")	
	self.Castbar.Icon = self.Castbar:CreateTexture(nil, "OVERLAY")
	self.Castbar.Icon:SetPoint("RIGHT", self.Castbar, "LEFT")
	self.Castbar.Icon:SetWidth(10)
	self.Castbar.Icon:SetHeight(10)
end

Skin.Func.TinyCastingBar = function(self)
	self.Castbar = CreateFrame("StatusBar")
	self.Castbar:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", insets ={left = -.8, right = -.8, top = -.8, bottom = -.8}})
	self.Castbar:SetBackdropColor(0, 0, 0)
	self.Castbar:SetWidth(self.db.width-10.8)
	self.Castbar:SetHeight(3)
	self.Castbar:SetStatusBarTexture(BarTexture2)	
	self.Castbar:SetStatusBarColor(unpack(oUF.colors.power.MANA))	 -- mana color[0]
	self.Castbar:SetParent(self)
	self.Castbar:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 10.8, .8)
	self.Castbar:SetMinMaxValues(1, 100)
	self.Castbar:SetValue(1)
	self.Castbar:Hide()  
	self.Castbar.bg = self.Castbar:CreateTexture(nil, "BORDER")
	self.Castbar.bg:SetAllPoints(self.Castbar)
	self.Castbar.bg:SetTexture(BarTexture2)
	self.Castbar.bg:SetAlpha(0.05)	
	self.Castbar.Spark = self:CreateTexture(nil, "OVERLAY")
	self.Castbar.Spark:SetParent(self.Castbar)
	self.Castbar.Spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	self.Castbar.Spark:SetBlendMode("ADD")
	self.Castbar.Spark:SetHeight(self.Castbar:GetHeight() * 2)
	self.Castbar.Spark:SetWidth(self.Castbar:GetHeight() * 2)
	self.Castbar.Time = self.Castbar:CreateFontString(nil, "OVERLAY")
	self.Castbar.Time:SetPoint("RIGHT", self.Castbar, -2, 0)
	self.Castbar.Time:SetFontObject(GameFontNormalSmall)
	self.Castbar.Time:SetTextColor(1, 1, 1)
	self.Castbar.Time:SetJustifyH("RIGHT")
	self.Castbar.Text = self.Castbar:CreateFontString(nil, "OVERLAY")
	self.Castbar.Text:SetPoint("TOPLEFT", self.Castbar, "BOTTOMLEFT", 2, 0)
	self.Castbar.Text:SetWidth(self.db.width/2)
	self.Castbar.Text:SetHeight(8)
	self.Castbar.Text:SetNonSpaceWrap(true)
	self.Castbar.Text:SetFontObject(GameFontNormalSmall)
	self.Castbar.Text:SetFont(MainFont, 8, "")
	self.Castbar.Text:SetTextColor(1, 1, 1)
	self.Castbar.Text:SetJustifyH"LEFT"
	self.Castbar.Icon = self.Castbar:CreateTexture(nil, "OVERLAY")
	self.Castbar.Icon:SetPoint("RIGHT", self.Castbar, "LEFT")
	self.Castbar.Icon:SetWidth(10)
	self.Castbar.Icon:SetHeight(10)
end

----------------------------------------------------------------------------
-- 		Dispell
----------------------------------------------------------------------------
local ICanDispell = function() return false end
do
	local _, class = UnitClass("player")
	if class == "DRUID" then
		ICanDispell = function(type) return (type == "Poison" or type == "Curse") end
	elseif class == "MAGE" then
		ICanDispell = function(type) return (type == "Curse") end
	elseif class == "SHAMAN" then
		ICanDispell = function(type) return (type == "Poison" or type == "Disease" or type == "Curse") end
	elseif class == "PALADIN" then
		ICanDispell = function(type) return (type == "Poison" or type == "Disease" or type == "Magic") end
	elseif class == "PRIEST" then
		ICanDispell = function(type) return (type == "Disease" or type == "Magic") end
	end
end
Skin.Func.ICanDispell = ICanDispell

Skin.Func.ImportantDebuffs = function(name, _, _, _, debuffType, _, expirationTime, isMine)
	return (
		   ICanDispell(debuffType)
		or name:match"Aimed"
		or name:match"Mortal"
		or name:match"Wound Poison"
		or name:match"Weakened"
		or name:match"Hypothermia"
		or name:match"Forbearance"
		or name:match"Recently Bandaged"
		or name:match"Mind-numbing Poison"
		)
end

Skin.Func.MyShortBuffs = function(name, _, _, _, _, duration, expirationTime, isMine)
	return (
		   (isMine and expirationTime < GetTime() + 30 and duration and duration > 0)
		or name:match"Last Stand"
		or name:match"Shield Wall"
		or name:match"Evasion"
		or name:match"Pain Suppression"
		)
end

----------------------------------------------------------------------------
-- 		Templates
----------------------------------------------------------------------------
local frameParent = function(self) return self:GetParent() end
local elementParent = function(self) return self end

local element = Skin.Elem
element.name = {
	font = MainFont,
	fontSize = MainFontSize,
	fontOutline = "",
	fontJustifyH = "LEFT",
	
	anchorTo = elementParent,
	anchorPoint = "LEFT",
	anchorRelative = "LEFT",
	anchorOffsetX = 3,
	anchorOffsetY = 0,
}

element.background = {
	alpha = .5,
	texture = BarTexture2,
	color = function(self) return 0, 0, 0 end,
}

element.health = {
	width = function(self) return self.db.width end,
	height = function(self) return self.db.height - 3.8 end, -- 3 px power bar
	
	texture = BarTexture1,
	
	anchorTo = elementParent,
	anchorPoint = "TOPLEFT",
	anchorRelative = "TOPLEFT",
	anchorOffsetX = 0,
	anchorOffsetY = 0,
	
	orientation = "HORIZONTAL",
	
	colorClass = true,
	colorTapping = true,
	colorReaction = true,
	colorDisconnected = true,
	
	enhanceEnable = true,
	enhanceSize = 10,
	enhanceAlpha = .3,
	enhanceTexture = StripeTexture,
}

element.afk = {
	width = 80,
	height = 8,

	font = MainFont,
	fontSize = MainFontSize*0.8,
	fontOutline = "",
	fontJustifyH = "CENTER",
	fontFormat = "|cFFFF0000AFK |cFFFFFFFF%s:%s",
	
	anchorTo = elementParent,
	anchorPoint = "CENTER",
	anchorRelative = "BOTTOM",
	anchorOffsetX = 0,
	anchorOffsetY = MainFontSize*0.4,
}

element.dnd = {
	width = 80,
	height = 8,

	font = MainFont,
	fontSize = MainFontSize*0.8,
	fontOutline = "",
	fontJustifyH = "CENTER",
	fontFormat = "|cFFFF5500DND |cFFFFFFFF%s:%s",
	
	anchorTo = elementParent,
	anchorPoint = "CENTER",
	anchorRelative = "BOTTOM",
	anchorOffsetX = 0,
	anchorOffsetY = 0,
}

element.healthtext = {
	font = MainFont,
	fontSize = MainFontSize,
	fontOutline = "",
	fontJustifyH = "LEFT",
	
	anchorTo = elementParent,
	anchorPoint = "RIGHT",
	anchorRelative = "RIGHT",
	anchorOffsetX = -3,
	anchorOffsetY = 0,
	
	onRefresh = Skin.Func.HealthTextHPMissing,
}

element.power = {
	width = function(self) return self.db.width end,
	height = 3, -- 3 px
	
	texture = BarTexture2,

	anchorTo = elementParent,
	anchorPoint = "BOTTOMLEFT",
	anchorRelative = "BOTTOMLEFT",
	anchorOffsetX = 0,
	anchorOffsetY = 0,
	
	orientation = "HORIZONTAL",
	
	colorPower = true,
	colorTapping = true,
	colorReaction = true,
	colorDisconnected = true,
	
	enhanceEnable = true,
	enhanceSize = 10,
	enhanceAlpha = .3,
	enhanceTexture = StripeTexture,
}

element.powertext = {
	font = MainFont,
	fontSize = MainFontSize,
	fontOutline = "",
	fontJustifyH = "LEFT",
	
	anchorTo = elementParent,
	anchorPoint = "BOTTOMRIGHT",
	anchorRelative = "BOTTOMRIGHT",
	anchorOffsetX = -3,
	anchorOffsetY = 0,
	
	onRefresh = Skin.Func.PowerTextLeft,
}


element.highlight = {
	alpha = .08,
	texture = function() return 1, 1, 1, 1 end,
}

element.raidicon = {
	width = function(self) return self.db.height/1.5 end, -- 1 px margin on each side
	height = function(self) return self.db.height/1.5 end, -- and 3 px power bar

	anchorTo = elementParent,
	anchorPoint = "CENTER",
	anchorRelative = "TOP",
	anchorOffsetX = 0,
	anchorOffsetY = 0,
}

element.leader = {
	width = function(self) return self.db.height/2 end,
	height = function(self) return self.db.height/2 end,

	anchorTo = elementParent,
	anchorPoint = "CENTER",
	anchorRelative = "TOPLEFT",
	anchorOffsetX = 0,
	anchorOffsetY = 0,
}

element.pvp = {
	width = function(self) return self.db.height/1.5 end,
	height = function(self) return self.db.height/1.5 end,

	anchorTo = elementParent,
	anchorPoint = "CENTER",
	anchorRelative = "LEFT",
	anchorOffsetX = 0,
	anchorOffsetY = 0,
}

element.combat = {
	width = function(self) return self.db.height/2 end,
	height = function(self) return self.db.height/2 end,

	anchorTo = elementParent,
	anchorPoint = "CENTER",
	anchorRelative = "TOPRIGHT",
	anchorOffsetX = 0,
	anchorOffsetY = 0,
	
	includeShade = false, -- Have the red border arround the swords?
}

element.rangecheck = {
	inRangeAlpha = 1,
	outOfRangeAlpha = .4,
}

element.healingfeedback = {
	postFunction = function(self)
		self.HealingFeedback = self:CreateFontString(nil, "OVERLAY")
		self.HealingFeedback:SetPoint("CENTER", self, "CENTER")
		self.HealingFeedback:SetFont(MainFont, MainFontSize*1.2, "Outline")
	end,
}
