-- Requires oUF
if not oUF or not oUF.Smurf then return end

local SmurfStripe = "Interface\\AddOns\\oUF_HolySmurf\\textures\\SmurfStripe"
local Minimalist  = "Interface\\AddOns\\oUF_HolySmurf\\textures\\Minimalist"
local PlainSmurf  = "Interface\\AddOns\\oUF_HolySmurf\\textures\\PlainSmurf"

----------------------------------------------------------------------------
-- 		Standard values
----------------------------------------------------------------------------
-- Just to make it simpler to change laods of units to the
-- samestandard values
----------------------------------------------------------------------------
local smalWidth     = 80
local smalHeight    = 20
local largeWidth    = 180
local largeHeight   = 30

-- Scales
local soloScale     = 1.0 -- Player, pet, target, targettarget, focus
local partyScale    = 1.0 -- Party**
local raidScale     = 1.0 -- Raid**

-- Offsets
local playerYOffset = -150 -- Distance from mid screen, also affects Target, TargetTarget, Focus and Pet
local onePix        = UIParent:GetEffectiveScale()

-- Pointers to templates.lua functions and elements
local SFunc         = oUF.HolySmurf.Func
local SElem         = oUF.HolySmurf.Elem
local copyTable     = oUF.Smurf.copyTable
local frameParent   = function(self) return self:GetParent() end
local elementParent = function(self) return self end
local ICanDispell   = oUF.HolySmurf.Func.ICanDispell

-- What should be shown?
local showSolo  = true -- Player, Target, TargetTarget, Focus, Pet
local showParty = true -- Party, PartyTarget, PartyPet
local showRaid  = true -- Raid, Maintank, Mainassist, Playertargets

----------------------------------------------------------------------------
-- 		Configuration
--	Prob going to move this to a seperate file
----------------------------------------------------------------------------
local config = {
	showPartyInRaid = false,
}

local units = oUF.HolySmurf.Config.units

----------------------------------------------------------------------------
-- 				SOLO
----------------------------------------------------------------------------
if showSolo then
----------------------------------
--	Player		--
----------------------------------
do
	local frame = {
		width = largeWidth,               -- This is one of the large frames
		height = largeHeight,             -- 
		scale = soloScale,
		anchorTo = "UIParent",            -- Independent frames are spawned at random, so you can't anchor them together, the exeption is "spawnFromParent" (look at PartyTarget).
		anchorPoint = "LEFT",             -- This frames left-middle sticks to:
		anchorRelative = "CENTER",        --       UIParent - Center => The center of your WoW window
		anchorOffsetX = -200,             -- Negative (down) 200 pixels
		anchorOffsetY = playerYOffset,    -- Default: -150, so Negative (left) 150 pixels
	}
	
	frame.element = {
		afk = copyTable(SElem.afk),
		pvp = copyTable(SElem.pvp),                     -- Copy everything from the pvp element in templates.lua
		name = copyTable(SElem.name),                   -- Copy everything .... name.....
		power = copyTable(SElem.power),                 -- Copy...
		health = copyTable(SElem.health),
		combat = copyTable(SElem.combat),
		leader = copyTable(SElem.leader),
		raidicon = copyTable(SElem.raidicon),
		highlight = copyTable(SElem.highlight),
		powertext = copyTable(SElem.powertext),
		background = copyTable(SElem.background),
		healthtext = copyTable(SElem.healthtext),
		castbar = { postFunction = SFunc.CastingBar },  -- if set, postFunction is run after all the current (castbar) element is spawned.
	}
	
	-- This good and all, but we want some of the values to differ from what they are set to in templates.lua
	frame.element.power.height = 8              -- Lets make the power bar bigger
	frame.element.power.texture = PlainSmurf    -- And change it to another texture
	
	frame.element.name.anchorOffsetY = 4                  -- The changes we made above, made the frame look odd, so lets move the name 4 pixels up
	frame.element.healthtext.anchorOffsetY = 4            -- The same goes for the health text
	frame.element.health.height = frame.height - 8 - onePix   -- The health bar needs to compensate so they don't overlap + a onePix px margin looks nice
	
	frame.element.resting = {
		postFunction = function(self)                                -- The postFunction is run after the rest of the settings are set, if any.
			self.Resting = self:CreateTexture(nil, "OVERLAY")   -- This function makes it easy to do what ever you'd like with this particular frame
			self.Resting:SetWidth(self.db.height/2)                  -- The self.db table is a reference to the "local frame" variable used here,
			self.Resting:SetHeight(self.db.height/2)                 -- so you have access to "everything you need".
			self.Resting:SetPoint("CENTER", self, "TOPRIGHT")
			self.Resting:SetTexture('Interface\\CharacterFrame\\UI-StateIcon')
			self.Resting:SetTexCoord(0.09, 0.43, 0.08, 0.42)
		end
	}
	
	do
		local aura = { }
		
		----------------------
		-- Buffs cluster
		----------------------
		aura.buff = {
			filter = "HELPFUL",
			growX = "RIGHT",
			growY = "DOWN",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.buff.cluster, {
			max = 10,
			size = 26,
			marginX = 3,
			marginY = 5,
			width = largeWidth,
			height = 32,
			anchorPoint = "TOPLEFT",
			anchorRelative = "BOTTOMLEFT",
			anchorOffsetX = -3,
			anchorOffsetY = -1,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.MyShortBuffs,
		})
		
		----------------------
		-- Debuffs cluster
		----------------------
		aura.debuff = {
			filter = "HARMFUL",
			growX = "RIGHT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.debuff.cluster, {
			max = 12,
			size = 26,
			marginX = 3,
			marginY = 5,
			width = largeWidth-smalWidth,
			height = 33,
			anchorPoint = "BOTTOMLEFT",
			anchorRelative = "TOPLEFT",
			anchorOffsetX = -3,
			anchorOffsetY = 11,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = function(_, _, _, _, type, _, expirationTime)
				return (expirationTime < GetTime() + 30)
			end
		})

		tinsert( aura.debuff.cluster, {
			max = 12,
			size = 16,
			marginX = 3,
			marginY = 5,
			width = largeWidth-smalWidth,
			height = 60,
			anchorPoint = "BOTTOMLEFT",
			anchorRelative = "TOPLEFT",
			anchorOffsetX = 0,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = function()
				return true
			end
		})

		frame.element.SmurfAura = {
			postFunction = function(self)
				self.SmurfAura = aura
			end
		}

	end
	
	units.player = frame   -- Move to the units table, witch in spawn.lua is sent to oUF_Smurf for possessing.
end

----------------------------------
--	Pet			--
----------------------------------
do
	local frame = {
		width = smalWidth,
		height = smalHeight,
		scale = soloScale,
		
		anchorTo = "UIParent",
		anchorPoint = "RIGHT",
		anchorRelative = "CENTER",
		anchorOffsetX = -200+largeWidth,
		anchorOffsetY = playerYOffset+largeHeight-3,
	}
		
	frame.element = {
		name = copyTable(SElem.name),
		power = copyTable(SElem.power),
		health = copyTable(SElem.health),
		raidicon = copyTable(SElem.raidicon),
		highlight = copyTable(SElem.highlight),
		background = copyTable(SElem.background),
		healthtext = copyTable(SElem.healthtext),
	}
	frame.element.name.anchorOffsetY = 2
	frame.element.health.texture = Minimalist
	
	do
		local aura = { }
		----------------------
		-- Debuffs cluster
		----------------------
		aura.debuff = {
			filter = "HARMFUL",
			growX = "RIGHT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.debuff.cluster, {
			max = 9,
			size = 16,
			marginX = 3,
			marginY = 5,
			width = smalWidth,
			height = 21,
			anchorPoint = "BOTTOMLEFT",
			anchorRelative = "TOPLEFT",
			anchorOffsetX = 0,
			anchorOffsetY = 1,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "TOP", 0, 0)
			end,
			Selector = SFunc.ImportantDebuffs,
		})

		frame.element.SmurfAura = {
			postFunction = function(self)
				self.SmurfAura = aura
			end
		}

	end

	units.pet = frame
end

----------------------------------
--	Focus		--
----------------------------------
do
	local frame = {
		width = smalWidth,
		height = smalHeight,
		scale = soloScale,
		
		anchorTo = "UIParent",
		anchorPoint = "RIGHT",
		anchorRelative = "CENTER",
		anchorOffsetX = -205,
		anchorOffsetY = playerYOffset,
	}

	frame.element = {
		pvp = copyTable(SElem.pvp),
		name = copyTable(SElem.name),
		power = copyTable(SElem.power),
		health = copyTable(SElem.health),
		leader = copyTable(SElem.leader),
		raidicon = copyTable(SElem.raidicon),
		highlight = copyTable(SElem.highlight),
		background = copyTable(SElem.background),
		healthtext = copyTable(SElem.healthtext),
		targetrangecheck = function(self) self.TargeRangeCheck = true end,
	}
	
	frame.element.name.onRefresh = SFunc.ShortName
	
	do
		local aura = { }
		
		----------------------
		-- Buffs cluster
		----------------------
		aura.buff = {
			filter = "HELPFUL",
			growX = "RIGHT",
			growY = "DOWN",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.buff.cluster, {
			max = 9,
			size = 16,
			marginX = 3,
			marginY = 5,
			width = smalWidth,
			height = 21,
			anchorPoint = "TOPLEFT",
			anchorRelative = "BOTTOMLEFT",
			anchorOffsetX = 0,
			anchorOffsetY = -1,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.MyShortBuffs,
		})
		
		----------------------
		-- Debuffs cluster
		----------------------
		aura.debuff = {
			filter = "HARMFUL",
			growX = "RIGHT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.debuff.cluster, {
			max = 9,
			size = 16,
			marginX = 3,
			marginY = 5,
			width = smalWidth,
			height = 21,
			anchorPoint = "BOTTOMLEFT",
			anchorRelative = "TOPLEFT",
			anchorOffsetX = 0,
			anchorOffsetY = 1,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "TOP", 0, 0)
			end,
			Selector = SFunc.ImportantDebuffs,
		})

		frame.element.SmurfAura = {
			postFunction = function(self)
				self.SmurfAura = aura
			end
		}

	end

	units.focus = frame
end

----------------------------------
--	Target		--
----------------------------------
do
	local frame = {
		width = largeWidth,
		height = largeHeight,
		scale = soloScale,
		
		anchorTo = "UIParent",
		anchorPoint = "RIGHT",
		anchorRelative = "CENTER",
		anchorOffsetX = 200,
		anchorOffsetY = playerYOffset,
	}

	frame.element = copyTable(units.focus.element)
	frame.element.powertext = copyTable(SElem.powertext)
	frame.element.powertext.onRefresh = SFunc.PowerTextTarget
	frame.element.castbar = { postFunction = SFunc.CastingBar }
	frame.element.healthtext.onRefresh = SFunc.HealthTextTarget
	frame.element.name.onRefresh = SFunc.TargetName
	frame.element.name.anchorOffsetY = 4
	frame.element.health.height = frame.height - 8 - onePix
	frame.element.healthtext.anchorOffsetY = 4
	frame.element.power.height = 8
	frame.element.power.texture = PlainSmurf
	
	do
		local aura = { }
		
		----------------------
		-- Buffs cluster
		----------------------
		aura.buff = {
			filter = "HELPFUL",
			growX = "RIGHT",
			growY = "DOWN",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.buff.cluster, {
			max = 10,
			size = 26,
			marginX = 3,
			marginY = 5,
			width = largeWidth,
			height = 32,
			anchorPoint = "TOPLEFT",
			anchorRelative = "BOTTOMLEFT",
			anchorOffsetX = -3,
			anchorOffsetY = -1,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.MyShortBuffs,
		})
		
		tinsert( aura.buff.cluster, {
			max = 6,
			size = 20,
			marginX = 3,
			marginY = 5,
			width = largeWidth,
			height = 32,
			anchorPoint = "TOPLEFT",
			anchorRelative = "BOTTOMLEFT",
			anchorOffsetX = 0,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.MyShortBuffs,
		})
		
		tinsert( aura.buff.cluster, {
			max = 20,
			size = 16,
			marginX = 3,
			marginY = 5,
			width = largeWidth,
			height = 60,
			anchorPoint = "TOPLEFT",
			anchorRelative = "BOTTOMLEFT",
			anchorOffsetX = 0,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = function()
				return true
			end
		})
		
		----------------------
		-- Debuffs cluster
		----------------------
		aura.debuff = {
			filter = "HARMFUL",
			growX = "RIGHT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.debuff.cluster, {
			max = 12,
			size = 26,
			marginX = 3,
			marginY = 5,
			width = largeWidth,
			height = 33,
			anchorPoint = "BOTTOMLEFT",
			anchorRelative = "TOPLEFT",
			anchorOffsetX = -3,
			anchorOffsetY = 11,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.MyShortBuffs,
		})

		tinsert( aura.debuff.cluster, {
			max = 12,
			size = 20,
			marginX = 3,
			marginY = 5,
			width = largeWidth,
			height = 60,
			anchorPoint = "BOTTOMLEFT",
			anchorRelative = "TOPLEFT",
			anchorOffsetX = 0,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.MyShortBuffs,
		})
		
		tinsert( aura.debuff.cluster, {
			max = 6,
			size = 16,
			marginX = 3,
			marginY = 5,
			width = largeWidth,
			height = 60,
			anchorPoint = "BOTTOMLEFT",
			anchorRelative = "TOPLEFT",
			anchorOffsetX = 0,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = function()
				return true
			end
		})

		frame.element.SmurfAura = {
			postFunction = function(self)
				self.SmurfAura = aura
			end
		}

	end
	
	units.target = frame
end

----------------------------------
--	TargetTarget	--
----------------------------------
do
	local frame = {
		width = smalWidth,
		height = smalHeight,
		scale = soloScale,
		
		anchorTo = "UIParent",
		anchorPoint = "LEFT",
		anchorRelative = "CENTER",
		anchorOffsetX = 205,
		anchorOffsetY = playerYOffset,
	}

	frame.element = {
		pvp = copyTable(SElem.pvp),
		name = copyTable(SElem.name),
		health = copyTable(SElem.health),
		leader = copyTable(SElem.leader),
		raidicon = copyTable(SElem.raidicon),
		highlight = copyTable(SElem.highlight),
		background = copyTable(SElem.background),
		healthtext = copyTable(SElem.healthtext),
		targetrangecheck = function(self) self.TargeRangeCheck = true end,
	}
	frame.element.name.onRefresh = SFunc.ShortName
	frame.element.health.height = frame.height
	frame.element.healthtext.onRefresh = SFunc.HealthTextHPPercent
	frame.element.SmurfAura = copyTable(units.focus.element.SmurfAura) -- Same auras as focus
	
	units.targettarget = frame
end

end -- showSolo
----------------------------------------------------------------------------
-- 				PARTY
----------------------------------------------------------------------------
if showParty then
----------------------------------
--	Party			--
----------------------------------
do
	local frame = {
		width = largeWidth,
		height = largeHeight + 1,
		scale = partyScale,
		
		anchorTo = "UIParent",
		anchorPoint = "TOPLEFT",
		anchorRelative = "BOTTOMLEFT",
		anchorOffsetX = 0,
		anchorOffsetY = -20,
	}

	frame.element = {
		pvp = copyTable(SElem.pvp),
		name = copyTable(SElem.name),
		power = copyTable(SElem.power),
		health = copyTable(SElem.health),
		leader = copyTable(SElem.leader),
		raidicon = copyTable(SElem.raidicon),
		highlight = copyTable(SElem.highlight),
		background = copyTable(SElem.background),
		rangecheck = copyTable(SElem.rangecheck),
		healthtext = copyTable(SElem.healthtext),
	}
	
	do
		local aura = { }
		
		----------------------
		-- Buffs cluster
		----------------------
		aura.buff = {
			filter = "HELPFUL",
			growX = "LEFT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.buff.cluster, {
			max = 5,
			size = 18,
			marginX = 3,
			marginY = 1,
			width = largeWidth,
			height = 24,
			anchorPoint = "TOPRIGHT",
			anchorRelative = "BOTTOMRIGHT",
			anchorOffsetX = 0,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.MyShortBuffs,
		})
		
		----------------------
		-- Debuffs cluster
		----------------------
		aura.debuff = {
			filter = "HARMFUL",
			growX = "RIGHT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.debuff.cluster, {
			max = 3,
			size = 18,
			marginX = 3,
			marginY = 1,
			width = largeWidth,
			height = 24,
			anchorPoint = "TOPLEFT",
			anchorRelative = "BOTTOMLEFT",
			anchorOffsetX = 0,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.ImportantDebuffs,
		})
		
		frame.element.SmurfAura = {
			postFunction = function(self)
				self.SmurfAura = aura
			end
		}

	end
	
	units.party = frame
end

----------------------------------
--	PartyTarget		--
----------------------------------
do
	local frame = {
		width = smalWidth,
		height = largeHeight / 2,
		--scale = partyScale, -- gets the same scale as it's parent: party
		
		spawnFromParent = true,  -- Important: This unitframe is not spawned by itself, but by it's parent. Only works for XXXtarget and XXXpet
		
		anchorTo = frameParent, -- Anchor to who ever spawned this frame
		anchorPoint = "BOTTOMLEFT",
		anchorRelative = "BOTTOMRIGHT",
		anchorOffsetX = 1,
		anchorOffsetY = 0,
	}
	frame.element = {
		pvp = copyTable(SElem.pvp),
		name = copyTable(SElem.name),
		health = copyTable(SElem.health),
		leader = copyTable(SElem.leader),
		raidicon = copyTable(SElem.raidicon),
		highlight = copyTable(SElem.highlight),
		background = copyTable(SElem.background),
		healthtext = copyTable(SElem.healthtext),
		targetrangecheck = function(self) self.TargeRangeCheck = true end,
	}
	frame.element.health.height = frame.height
	frame.element.healthtext.onRefresh = SFunc.HealthTextHPPercent
	
	units.partytarget = frame
end

----------------------------------
--	PartyPet		--
----------------------------------
do
	local frame = {
		width = smalWidth,
		height = largeHeight / 2,
		--scale = partyScale, -- gets the same scale as it's parent: party
		
		spawnFromParent = true, -- Important: This unitframe is not spawned by itself, but by it's parent. Only works for XXXtarget and XXXpet
		
		anchorTo = frameParent, -- Anchor to a spesific party member
		anchorPoint = "TOPLEFT",
		anchorRelative = "TOPRIGHT",
		anchorOffsetX = 1,
		anchorOffsetY = 0,
	}

	frame.element = copyTable(units.partytarget.element)
	
	do
		local aura = { }
		
		----------------------
		-- Buffs cluster
		----------------------
		aura.buff = {
			filter = "HELPFUL",
			growX = "RIGHT",
			growY = "DOWN",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.buff.cluster, {
			max = 9,
			size = 16,
			marginX = 3,
			marginY = 5,
			width = smalWidth,
			height = 21,
			anchorPoint = "TOPLEFT",
			anchorRelative = "TOPRIGHT",
			anchorOffsetX = 0,
			anchorOffsetY = -1,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.MyShortBuffs,
		})
		
		----------------------
		-- Debuffs cluster
		----------------------
		aura.debuff = {
			filter = "HARMFUL",
			growX = "RIGHT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.debuff.cluster, {
			max = 9,
			size = 16,
			marginX = 3,
			marginY = 5,
			width = smalWidth,
			height = 21,
			anchorPoint = "BOTTOMLEFT",
			anchorRelative = "BOTTOMRIGHT",
			anchorOffsetX = 0,
			anchorOffsetY = 1,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "TOP", 0, 0)
			end,
			Selector = SFunc.ImportantDebuffs,
		})

		frame.element.SmurfAura = {
			postFunction = function(self)
				self.SmurfAura = aura
			end
		}

	end
	
	units.partypet = frame
end

end -- showParty
----------------------------------------------------------------------------
-- 				RAID
----------------------------------------------------------------------------
if showRaid then
----------------------------------
--	Raid			--
----------------------------------
do
	local frame = {
		width = smalWidth,
		height = smalHeight,
		scale = raidScale,
		
		spawnFromParent = true,
		
		anchorTo = "UIParent",
		anchorPoint = "TOPLEFT",
		anchorRelative = "TOPRIGHT",
		anchorOffsetX = 1,
		anchorOffsetY = 0,
	}

	frame.element = {
		afk = copyTable(SElem.afk),
		pvp = copyTable(SElem.pvp),
		name = copyTable(SElem.name),
		power = copyTable(SElem.power),
		health = copyTable(SElem.health),
		leader = copyTable(SElem.leader),
		raidicon = copyTable(SElem.raidicon),
		highlight = copyTable(SElem.highlight),
		background = copyTable(SElem.background),
		rangecheck = copyTable(SElem.rangecheck),
		healthtext = copyTable(SElem.healthtext),
	}
	frame.element.name.onRefresh = SFunc.ShortName
	frame.element.name.anchorOffsetY = 2 -- Make room for buffs
	frame.element.healthtext.anchorOffsetY = 2 -- Make room for buffs
	
	do
		local aura = { }
		
		----------------------
		-- Buffs cluster
		----------------------
		aura.buff = {
			filter = "HELPFUL",
			growX = "LEFT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.buff.cluster, {
			max = 3,
			size = 14,
			marginX = 3,
			marginY = 1,
			width = smalWidth,
			height = 16,
			anchorPoint = "TOPRIGHT",
			anchorRelative = "BOTTOMRIGHT",
			anchorOffsetX = 3,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.MyShortBuffs,
		})
		
		----------------------
		-- Debuffs cluster
		----------------------
		aura.debuff = {
			filter = "HARMFUL",
			growX = "RIGHT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.debuff.cluster, {
			max = 3,
			size = 14,
			marginX = 3,
			marginY = 1,
			width = smalWidth,
			height = 16,
			anchorPoint = "TOPLEFT",
			anchorRelative = "BOTTOMLEFT",
			anchorOffsetX = -3,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.ImportantDebuffs,
		})
		
		frame.element.SmurfAura = {
			postFunction = function(self)
				self.SmurfAura = aura
			end
		}

	end
	
	units.raid = frame
end

----------------------------------
--	RaidPet		--
----------------------------------
do
	local frame = {
		width = smalWidth,
		height = smalHeight/3,
		--scale = raidScale, -- gets the same scale as it's parent: party
		
		spawnFromParent = true,
		
		anchorTo = frameParent,
		anchorPoint = "TOPLEFT",
		anchorRelative = "BOTTOMLEFT",
		anchorOffsetX = 0,
		anchorOffsetY = -1,
	}

	frame.element = {
		health = copyTable(SElem.health),
		raidicon = copyTable(SElem.raidicon),
		highlight = copyTable(SElem.highlight),
		background = copyTable(SElem.background),
		rangecheck = copyTable(SElem.rangecheck),
		healthtext = copyTable(SElem.healthtext),
	}
	frame.element.health.height = frame.height
	frame.element.health.texture = Minimalist
	units.raidpet = frame
end

----------------------------------
--	MainTank		--
----------------------------------
do
	local frame = {
		width = smalWidth,
		height = smalHeight,
		scale = raidScale,
		
		anchorTo = "UIParent",
		anchorPoint = "LEFT",
		anchorRelative = "LEFT",
		anchorOffsetX = 0,
		anchorOffsetY = 0,
	}

	frame.element = copyTable(units.raid.element)
	frame.element.rangecheck = nil
	frame.element.targetrangecheck = function(self) self.TargeRangeCheck = true end
	frame.element.afk = copyTable(SElem.afk)
	frame.element.power = nil
	frame.element.leader = nil
	frame.element.name.onRefresh = SFunc.ShortName
	frame.element.name.anchorRelative = "TOPLEFT"
	frame.element.health.height = frame.height
	frame.element.name.anchorOffsetY = 2 -- Make room for buffs
	frame.element.healthtext.anchorOffsetY = 2 -- Make room for buffs
	do
		local aura = { }
		
		----------------------
		-- Buffs cluster
		----------------------
		aura.buff = {
			filter = "HELPFUL",
			growX = "LEFT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.buff.cluster, {
			max = 3,
			size = 16,
			marginX = 3,
			marginY = 0,
			width = smalWidth,
			height = 16,
			anchorPoint = "RIGHT",
			anchorRelative = "LEFT",
			anchorOffsetX = 0,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.MyShortBuffs,
		})
		
		----------------------
		-- Debuffs cluster
		----------------------
		aura.debuff = {
			filter = "HARMFUL",
			growX = "RIGHT",
			growY = "UP",
			cluster = { },
			PostCreateIcon = SFunc.PostCreateAuraIcon,
		}

		tinsert( aura.debuff.cluster, {
			max = 3,
			size = 16,
			marginX = 3,
			marginY = 0,
			width = smalWidth,
			height = 16,
			anchorPoint = "LEFT",
			anchorRelative = "LEFT",
			anchorOffsetX = 0,
			anchorOffsetY = 0,
			PostCreateCooldown = function(timer)
				timer.text:ClearAllPoints()
				timer.text:SetPoint("CENTER", timer, "BOTTOM", 0, 0)
			end,
			Selector = SFunc.ImportantDebuffs,
		})
		
		frame.element.SmurfAura = {
			postFunction = function(self)
				self.SmurfAura = aura
			end
		}

	end
	
	units.maintank = frame
end

----------------------------------
--	MainTankTarget	--
----------------------------------
do
	local frame = {
		width = smalWidth,
		height = smalHeight,
		--scale = raidScale, -- gets the same scale as it's parent: party
		
		spawnFromParent = true,
		
		anchorTo = frameParent,
		anchorPoint = "LEFT",
		anchorRelative = "RIGHT",
		anchorOffsetX = 1,
		anchorOffsetY = 0,
	}

	frame.element = {
		name = copyTable(SElem.name),
		health = copyTable(SElem.health),
		raidicon = copyTable(SElem.raidicon),
		highlight = copyTable(SElem.highlight),
		background = copyTable(SElem.background),
		healthtext = copyTable(SElem.healthtext),
		targetrangecheck = function(self) self.TargeRangeCheck = true end,
		castbar = { postFunction = SFunc.TinyCastingBar },
	}
	frame.element.name.onRefresh = SFunc.ShortName
	frame.element.health.height = frame.height
	frame.element.healthtext.onRefresh = SFunc.HealthTextHPPercent	
	units.maintanktarget = frame
end

----------------------------------
--	MainAssist		--
----------------------------------
do
	local frame = copyTable(units.maintank)
	units.mainassist = frame
end

----------------------------------
--	MainAssistTarget	--
----------------------------------
do
	local frame = copyTable(units.maintanktarget)
	units.mainassisttarget = frame
end

----------------------------------
--	PlayerTarget	--
----------------------------------
do
	local frame = copyTable(units.maintank)
	units.raidplayertarget = frame
end

----------------------------------
--	PlayerTargetTarget
----------------------------------
do
	local frame = copyTable(units.maintanktarget)
	units.raidplayertargettarget = frame
end

end -- showRaid
----------------------------------------------------------------------------
-- 				Append to all units
----------------------------------------------------------------------------
for key,frame in pairs(units) do
	frame.element.healingfeedback = SElem.healingfeedback
end

----------------------------------------------------------------------------
-- 				Save
----------------------------------------------------------------------------
config.units = units
oUF.SmurfConfig = config
