-- Exit if oUF isn't loaded
if not oUF then return end
local tinsert, floor, pairs, ipairs =
	  table.insert, math.floor, pairs, ipairs
local UnitAura = UnitAura
local DebuffTypeColor = DebuffTypeColor
local debug = false

local objects = { }

local ResetClusters = function(clusters)
	for _,cluster in ipairs(clusters) do
		cluster.numShownPreviously = cluster.numShown or -1
		cluster.numShown = 0
	end
end

local UpdateClusterSize = function(cluster)
	--[[ DEBUG
	debug = (cluster.frame.unit=="target")
	if debug then print("update cluster: "..cluster.num.." for "..cluster.frame.unit) end
	debug = false
	--]]
	if cluster.numShown == 0 then
		cluster:SetWidth(0.001)
		cluster:SetHeight(0.001)
		--[[ DEBUG
		if debug then print"|cFFFF0000Hide" end
		--]]
	else
		local i = cluster.numShown
		local xPos, yPos = i, 1
		
		if i > cluster.prRow then
			yPos = floor(i/cluster.prRow) + 1
			xPos = i - (yPos-1)*cluster.prRow
		end
		--[[ DEBUG
		if debug then print("|cFF00FF00Pos: "..xPos.."x"..yPos) end
		--]]
		if xPos == 1 or xPos == cluster.prRow or cluster.forceUpdateSize then
			cluster:SetHeight(yPos * cluster.db.size + (yPos + 1) * cluster.db.marginY)
		end
		if yPos == 1 or cluster.forceUpdateSize then
			cluster:SetWidth(xPos * cluster.db.size + (xPos + 1) * cluster.db.marginX)
		end
	end
end

local CleanupClusters = function(clusters)
	for _,cluster in ipairs(clusters) do
		if cluster.numShownPreviously > cluster.numShown then
			for i=1, cluster.numShownPreviously - cluster.numShown do
				cluster.icon[cluster.numShownPreviously-i+1]:Hide() -- Hide from high to low buff index
			end
		end
		if cluster.numShownPreviously ~= cluster.numShown or cluster.forceUpdateSize then
			UpdateClusterSize(cluster)
		end
	end
end

local OnEnter = function(self)
	if(not self:IsVisible()) then return end

	GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT")
	GameTooltip:SetUnitAura(self.frame.unit, self.id, self.filter)
end

local OnLeave = function()
	GameTooltip:Hide()
end

local CreateAuraIcon = function(self, group, cluster)
	if cluster.icon[cluster.numShown] then return cluster.icon[cluster.numShown] end
	
	-- Create it
	local i = cluster.numShown - 1 -- we start at 0
	
	if i == 0 then
		cluster.xDir = (group.growX == "RIGHT" and 1) or -1
		cluster.yDir = (group.growY == "UP" and 1) or -1
		cluster.initAnchor = ((group.growY == "UP" and "BOTTOM") or "TOP")..((group.growX == "RIGHT" and "LEFT") or "RIGHT")
		cluster.prRow = floor( (cluster.db.width + cluster.db.marginX) / (cluster.db.size + cluster.db.marginX) )
		local maxRows = floor( (cluster.db.height + cluster.db.marginY) / (cluster.db.size + cluster.db.marginY) )
		if cluster.db.max > maxRows * cluster.prRow then
			cluster.db.max = maxRows * cluster.prRow
			if cluster.db.max < 1 then cluster.db.max = 1 end -- Min 1
			--print("|cFFFF0000Reset max to: "..cluster.db.max.." for "..cluster.num)
		end
	end
	
	local x, y
	local xPos, yPos = i, 0
	
	if i >= cluster.prRow then
		yPos = floor(i/cluster.prRow)
		xPos = i - yPos*cluster.prRow
	end
	
	x = (xPos * cluster.db.size + (xPos + 1) * cluster.db.marginX) * cluster.xDir
	y = (yPos * cluster.db.size + (yPos + 1) * cluster.db.marginY) * cluster.yDir
	
	local aura = CreateFrame("Frame", nil, cluster)
	aura:SetPoint(cluster.initAnchor, cluster, cluster.initAnchor, x, y)
	aura:EnableMouse(true)

	aura:SetWidth(cluster.db.size or 16)
	aura:SetHeight(cluster.db.size or 16)

	local cd = CreateFrame("Cooldown", nil, aura)
	cd:SetAllPoints(aura)
	cd.PostCreateCooldown = cluster.db.PostCreateCooldown

	local icon = aura:CreateTexture(nil, "BACKGROUND")
	icon:SetAllPoints(aura)

	local count = aura:CreateFontString(nil, "OVERLAY")
	count:SetFontObject(NumberFontNormal)
	count:SetPoint("BOTTOMRIGHT", aura, "BOTTOMRIGHT", 0, 0)

	local overlay = aura:CreateTexture(nil, "OVERLAY")
	overlay:SetTexture"Interface\\Buttons\\UI-Debuff-Overlays"
	overlay:SetAllPoints(aura)
	overlay:SetTexCoord(.296875, .5703125, 0, .515625)
	aura.overlay = overlay

	aura:SetScript("OnEnter", OnEnter)
	aura:SetScript("OnLeave", OnLeave)

	aura.parent = cluster
	aura.frame = self
	aura.debuff = group.filter:match"HARMFUL"
	aura.filter = group.filter

	aura.icon = icon
	aura.count = count
	aura.cd = cd

	if(group.PostCreateIcon) then group.PostCreateIcon(self, aura, cluster, cluster.numShown, group.filter:match"HARMFUL") end
	cluster.icon[cluster.numShown] = aura
	
	return aura
end
	
local GetAura = function(self, group, cluster)
	cluster.numShown = cluster.numShown + 1
	local aura = cluster.icon[cluster.numShown] or CreateAuraIcon(self, group, cluster)
	if not aura:IsShown() then aura:Show() end
	return aura
end

local UpdateAuras
do
	local i
	local unit, filter, isUsed, aura
	local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable
	
	local NextAura = function() -- Populates the return values from UnitAura, returns nil when iteration ends.
		i = i + 1
		name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitAura(unit, i, filter)
		return name
	end
	
	UpdateAuras = function(self, event, eventUnit)
		if not(eventUnit == self.unit -- Event unit and this frames unit must match
		   and self:IsShown()         -- This frame must be visible
		   and self.SmurfAuraGroup)   -- This frame must implement SmurfAuras
		   then return end
		   
		unit = eventUnit
		for _,group in pairs(self.SmurfAuraGroup) do
			i = 0
			filter = group.db.filter
			ResetClusters(group.cluster)
			while(NextAura()) do
				isUsed = false
				for _,cluster in ipairs(group.cluster) do
					if not isUsed and cluster.numShown < cluster.db.max then
						if cluster.db.Selector(name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable) then
							isUsed = true
							aura = GetAura(self, group.db, cluster)
							aura.id = i
							aura.icon:SetTexture(icon)             -- Icon
							aura.count:SetText(count > 1 and count) -- blank if 1 or nil
							-- Duration timer
							if(duration and duration > 0) then
								aura.cd:SetCooldown(expirationTime - duration, duration)
								aura.cd:Show()
							else
								aura.cd:Hide()
							end
							
							-- Bordercolor
							if debuffType and DebuffTypeColor[debuffType] then
								local color = DebuffTypeColor[debuffType]
								aura.overlay:SetVertexColor(color.r, color.g, color.b)
							elseif filter:match"HARMFUL" then
								aura.overlay:SetVertexColor(1, 0, 0)
							else
								aura.overlay:SetVertexColor(.3, .3, .3)
							end
						end
					end
				end
			end
			CleanupClusters(group.cluster)
		end
	end
end

local OnEvent = function(eventFrame, event, ...)
	UpdateAuras(eventFrame:GetParent(), ...)
end

local OnShow = function(eventFrame)
	if eventFrame:GetParent().unit == "target" then debug = true end
	--[[ DEBUG
	if debug then print("SHOW"..eventFrame:GetParent().unit) end
	--]]
	for _,cluster in ipairs(eventFrame.group.cluster) do
		cluster.forceUpdateSize = true
	end
	UpdateAuras(eventFrame:GetParent(), "OnShow", eventFrame:GetParent().unit)
	--[[ DEBUG
	debug = false
	--]]
end

local hook = function(self)
	if not self.SmurfAura then return end
	self.SmurfAuraGroup = { }
	for name,grp in pairs(self.SmurfAura) do
		local group = { }
		group.cluster = { }
		group.db = grp
		
		local last = nil
		for i,clus in ipairs(grp.cluster) do
			local cluster = CreateFrame("Frame", nil, self)
			cluster:SetFrameLevel(self:GetFrameLevel() + 2)
			cluster:SetWidth(clus.size + 2 * clus.marginX)  -- temp fix =/
			cluster:SetHeight(clus.size + 2 * clus.marginY) --
			cluster.db = clus
			cluster.num = i
			--[[ DEBUG
			cluster.frame = self
			--]]
			
			cluster.numShown = 0
			cluster.icon = { }
			--[[ DEBUG, shows each cluster in a different color, (up to 3)
			cluster.texture = cluster:CreateTexture(nil, "BACKGROUND")
			cluster.texture:SetAllPoints(cluster)
			if i == 1 then
				cluster.texture:SetTexture(1, 0, 0, .3)
			elseif i == 2 then
				cluster.texture:SetTexture(0, 1, 0, .3)
			else
				cluster.texture:SetTexture(0, 0, 1, .3)
			end
			--]]
			
			cluster:SetPoint(
				clus.anchorPoint or "LEFT",
				(clus.anchorToFrame and self) or last or self,
				clus.anchorRelative or "RIGHT",
				clus.anchorOffsetX or 0,
				clus.anchorOffsetY or 0)
				
			if i == 1 then
				cluster:SetScript("OnShow", OnShow)
				cluster.group = group
			end
				
			last = cluster
			tinsert(group.cluster, cluster)
		end

		self.SmurfAuraGroup[name] = group
	end

	self.UNIT_AURA = UpdateAuras
	self:RegisterEvent("UNIT_AURA")
	
	tinsert(objects, self)
end

-- Append to all existing frames
for _,object in pairs(oUF.objects) do hook(object) end

-- Append to frames created later on
oUF:RegisterInitCallback(hook)

oUF:RegisterSubTypeMapping"UNIT_AURA"
